@@ -4,13 +4,15 @@ use std::fmt;
 use std::marker::PhantomData;
 
 use cgl::CGLSetParameter;
-use icrate::AppKit::{NSOpenGLCPSwapInterval, NSView};
-use icrate::Foundation::{MainThreadBound, MainThreadMarker};
 use objc2::rc::{autoreleasepool, Id};
 use objc2::ClassType;
+use objc2_app_kit::{NSOpenGLCPSwapInterval, NSView};
+use objc2_foundation::{run_on_main, MainThreadBound};
 
 use crate::config::GetGlConfig;
-use crate::context::{AsRawContext, ContextApi, ContextAttributes, RawContext, Robustness};
+use crate::context::{
+    AsRawContext, ContextApi, ContextAttributes, Priority, RawContext, Robustness,
+};
 use crate::display::GetGlDisplay;
 use crate::error::{ErrorKind, Result};
 use crate::prelude::*;
@@ -74,6 +76,13 @@ pub struct NotCurrentContext {
 }
 
 impl NotCurrentContext {
+    /// Make a [`Self::PossiblyCurrentContext`] indicating that the context
+    /// could be current on the thread.
+    pub fn make_current_surfaceless(self) -> Result<PossiblyCurrentContext> {
+        self.inner.make_current_surfaceless()?;
+        Ok(PossiblyCurrentContext { inner: self.inner, _nosendsync: PhantomData })
+    }
+
     fn new(inner: ContextInner) -> Self {
         Self { inner, _nosync: PhantomData }
     }
@@ -108,6 +117,10 @@ impl GlContext for NotCurrentContext {
     fn context_api(&self) -> ContextApi {
         self.inner.context_api()
     }
+
+    fn priority(&self) -> Priority {
+        Priority::Medium
+    }
 }
 
 impl GetGlConfig for NotCurrentContext {
@@ -143,15 +156,26 @@ pub struct PossiblyCurrentContext {
     _nosendsync: PhantomData<*mut ()>,
 }
 
+impl PossiblyCurrentContext {
+    /// Make this context current on the calling thread.
+    pub fn make_current_surfaceless(&self) -> Result<()> {
+        self.inner.make_current_surfaceless()
+    }
+}
+
 impl PossiblyCurrentGlContext for PossiblyCurrentContext {
     type NotCurrentContext = NotCurrentContext;
     type Surface<T: SurfaceTypeTrait> = Surface<T>;
 
     fn make_not_current(self) -> Result<Self::NotCurrentContext> {
-        self.inner.make_not_current()?;
+        self.make_not_current_in_place()?;
         Ok(NotCurrentContext::new(self.inner))
     }
 
+    fn make_not_current_in_place(&self) -> Result<()> {
+        self.inner.make_not_current()
+    }
+
     fn is_current(&self) -> bool {
         if let Some(current) = NSOpenGLContext::currentContext() {
             current == self.inner.raw
@@ -177,6 +201,10 @@ impl GlContext for PossiblyCurrentContext {
     fn context_api(&self) -> ContextApi {
         self.inner.context_api()
     }
+
+    fn priority(&self) -> Priority {
+        Priority::Medium
+    }
 }
 
 impl GetGlConfig for PossiblyCurrentContext {
@@ -224,7 +252,7 @@ impl ContextInner {
             self.raw.makeCurrentContext();
 
             let view = &surface.ns_view;
-            MainThreadMarker::run_on_main(|mtm| unsafe {
+            run_on_main(|mtm| unsafe {
                 self.raw.setView(Some(view.get(mtm)));
             });
 
@@ -232,6 +260,19 @@ impl ContextInner {
         })
     }
 
+    fn make_current_surfaceless(&self) -> Result<()> {
+        autoreleasepool(|_| {
+            self.update();
+            self.raw.makeCurrentContext();
+
+            run_on_main(|_mtm| unsafe {
+                self.raw.setView(None);
+            });
+
+            Ok(())
+        })
+    }
+
     fn context_api(&self) -> ContextApi {
         ContextApi::OpenGl(None)
     }
@@ -248,7 +289,7 @@ impl ContextInner {
     }
 
     pub(crate) fn update(&self) {
-        MainThreadMarker::run_on_main(|_| self.raw.update());
+        run_on_main(|_| self.raw.update());
     }
 
     pub(crate) fn flush_buffer(&self) -> Result<()> {
@@ -259,7 +300,7 @@ impl ContextInner {
     }
 
     pub(crate) fn is_view_current(&self, view: &MainThreadBound<Id<NSView>>) -> bool {
-        MainThreadMarker::run_on_main(|mtm| {
+        run_on_main(|mtm| {
             self.raw.view(mtm).expect("context to have a current view") == *view.get(mtm)
         })
     }
