@@ -2,18 +2,18 @@
 
 use std::collections::HashSet;
 use std::ffi::{self, CStr};
+use std::fmt;
 use std::mem::MaybeUninit;
 use std::ops::Deref;
 use std::os::raw::c_char;
 use std::sync::Arc;
-use std::{fmt, ptr};
 
 use glutin_egl_sys::egl;
 use glutin_egl_sys::egl::types::{EGLAttrib, EGLDisplay, EGLint};
 
 use once_cell::sync::OnceCell;
 
-use raw_window_handle::RawDisplayHandle;
+use raw_window_handle::{RawDisplayHandle, XlibDisplayHandle};
 
 use crate::config::ConfigTemplate;
 use crate::context::Version;
@@ -89,8 +89,10 @@ impl Display {
     ///
     /// # Safety
     ///
-    /// If `raw_display` is [`Some`], `raw_display` must point to a valid system
-    /// display.
+    /// If `raw_display` is [`Some`], `raw_display` must point to a valid
+    /// [`RawDisplayHandle::Drm`]. The provided
+    /// [`raw_display_handle::DrmDisplayHandle.fd`] may be closed after calling
+    /// this function.
     pub unsafe fn with_device(
         device: &Device,
         raw_display: Option<RawDisplayHandle>,
@@ -100,13 +102,6 @@ impl Display {
             None => return Err(ErrorKind::NotFound.into()),
         };
 
-        if raw_display.is_some() {
-            return Err(ErrorKind::NotSupported(
-                "Display::with_device does not support a `raw_display` argument yet",
-            )
-            .into());
-        }
-
         if !egl.GetPlatformDisplayEXT.is_loaded() {
             return Err(ErrorKind::NotSupported("eglGetPlatformDisplayEXT is not supported").into());
         }
@@ -127,9 +122,22 @@ impl Display {
 
         let mut attrs = Vec::<EGLint>::with_capacity(3);
 
-        // TODO: Some extensions exist like EGL_EXT_device_drm which allow specifying
-        // which DRM master fd to use under the hood by the implementation. This would
-        // mean there would need to be an unsafe equivalent to this function.
+        match raw_display {
+            Some(RawDisplayHandle::Drm(handle))
+                if device.extensions().contains("EGL_EXT_device_drm") =>
+            {
+                attrs.push(egl::DRM_MASTER_FD_EXT as EGLint);
+                attrs.push(handle.fd as EGLint);
+            },
+            Some(_) => {
+                return Err(ErrorKind::NotSupported(
+                    "`egl::display::Display::with_device()` does not support \
+                     non-`DrmDisplayHandle` `RawDisplayHandle`s",
+                )
+                .into())
+            },
+            None => {},
+        };
 
         // Push at the end so we can pop it on failure
         let mut has_display_reference = extensions.contains("EGL_KHR_display_reference");
@@ -191,12 +199,12 @@ impl Display {
             .into());
         }
 
-        let device = ptr::null_mut();
+        let mut device = MaybeUninit::uninit();
         if unsafe {
             self.inner.egl.QueryDisplayAttribEXT(
                 *self.inner.raw,
                 egl::DEVICE_EXT as EGLint,
-                device as *mut _,
+                device.as_mut_ptr(),
             )
         } == egl::FALSE
         {
@@ -209,9 +217,21 @@ impl Display {
             }));
         }
 
+        let device = unsafe { device.assume_init() } as egl::types::EGLDeviceEXT;
+        debug_assert_ne!(
+            device,
+            egl::NO_DEVICE_EXT,
+            "eglQueryDisplayAttribEXT(EGL_DEVICE_EXT) should never return EGL_NO_DEVICE_EXT on \
+             success"
+        );
         Device::from_ptr(self.inner.egl, device)
     }
 
+    /// Get a reference to the initialized EGL API.
+    pub fn egl(&self) -> &'static Egl {
+        self.inner.egl
+    }
+
     /// Terminate the EGL display.
     ///
     /// When the display is managed by glutin with the
@@ -239,21 +259,28 @@ impl Display {
         let extensions = CLIENT_EXTENSIONS.get().unwrap();
 
         let mut attrs = Vec::<EGLAttrib>::with_capacity(5);
-        let (platform, mut display) = match display {
-            #[cfg(wayland_platform)]
+        let (platform, display) = match display {
             RawDisplayHandle::Wayland(handle)
                 if extensions.contains("EGL_KHR_platform_wayland") =>
             {
-                (egl::PLATFORM_WAYLAND_KHR, handle.display)
+                (egl::PLATFORM_WAYLAND_KHR, handle.display.as_ptr())
             },
-            #[cfg(x11_platform)]
             RawDisplayHandle::Xlib(handle) if extensions.contains("EGL_KHR_platform_x11") => {
                 attrs.push(egl::PLATFORM_X11_SCREEN_KHR as EGLAttrib);
                 attrs.push(handle.screen as EGLAttrib);
-                (egl::PLATFORM_X11_KHR, handle.display)
+                (
+                    egl::PLATFORM_X11_KHR,
+                    handle.display.map_or(egl::DEFAULT_DISPLAY as *mut _, |d| d.as_ptr()),
+                )
             },
             RawDisplayHandle::Gbm(handle) if extensions.contains("EGL_KHR_platform_gbm") => {
-                (egl::PLATFORM_GBM_KHR, handle.gbm_device)
+                (egl::PLATFORM_GBM_KHR, handle.gbm_device.as_ptr())
+            },
+            RawDisplayHandle::Drm(_) => {
+                return Err(ErrorKind::NotSupported(
+                    "`DrmDisplayHandle` must be used with `egl::display::Display::with_device()`",
+                )
+                .into())
             },
             RawDisplayHandle::Android(_) if extensions.contains("EGL_KHR_platform_android") => {
                 (egl::PLATFORM_ANDROID_KHR, egl::DEFAULT_DISPLAY as *mut _)
@@ -265,11 +292,6 @@ impl Display {
             },
         };
 
-        // Be explicit here.
-        if display.is_null() {
-            display = egl::DEFAULT_DISPLAY as *mut _;
-        }
-
         // Push at the end so we can pop it on failure
         let mut has_display_reference = extensions.contains("EGL_KHR_display_reference");
         if has_display_reference {
@@ -315,30 +337,48 @@ impl Display {
 
         let mut attrs = Vec::<EGLint>::with_capacity(5);
         let mut legacy = false;
-        let (platform, mut display) = match display {
-            #[cfg(wayland_platform)]
+        let (platform, display) = match display {
             RawDisplayHandle::Wayland(handle)
                 if extensions.contains("EGL_EXT_platform_wayland") =>
             {
-                (egl::PLATFORM_WAYLAND_EXT, handle.display)
+                (egl::PLATFORM_WAYLAND_EXT, handle.display.as_ptr())
             },
-            #[cfg(x11_platform)]
             RawDisplayHandle::Xlib(handle) if extensions.contains("EGL_EXT_platform_x11") => {
                 attrs.push(egl::PLATFORM_X11_SCREEN_EXT as EGLint);
                 attrs.push(handle.screen as EGLint);
-                (egl::PLATFORM_X11_EXT, handle.display)
+                (
+                    egl::PLATFORM_X11_EXT,
+                    handle.display.map_or(egl::DEFAULT_DISPLAY as *mut _, |d| d.as_ptr()),
+                )
             },
-            #[cfg(x11_platform)]
             RawDisplayHandle::Xcb(handle)
                 if extensions.contains("EGL_MESA_platform_xcb")
                     || extensions.contains("EGL_EXT_platform_xcb") =>
             {
                 attrs.push(egl::PLATFORM_XCB_SCREEN_EXT as EGLint);
                 attrs.push(handle.screen as EGLint);
-                (egl::PLATFORM_XCB_EXT, handle.connection)
+                (
+                    egl::PLATFORM_XCB_EXT,
+                    handle.connection.map_or(egl::DEFAULT_DISPLAY as *mut _, |c| c.as_ptr()),
+                )
             },
-            RawDisplayHandle::Gbm(handle) if extensions.contains("EGL_MESA_platform_gbm") => {
-                (egl::PLATFORM_GBM_MESA, handle.gbm_device)
+            RawDisplayHandle::Gbm(handle)
+                // NOTE: Some drivers report that they support the KHR GBM extension without EGL
+                // 1.5 client, so work around that here by checking the KHR GBM extension as well.
+                // The MESA and KHR extensions have the same constant values, thus it'll work
+                // regardless.
+                //
+                // See https://github.com/rust-windowing/glutin/issues/1708.
+                if extensions.contains("EGL_MESA_platform_gbm")
+                    || extensions.contains("EGL_KHR_platform_gbm") =>
+            {
+                (egl::PLATFORM_GBM_MESA, handle.gbm_device.as_ptr())
+            },
+            RawDisplayHandle::Drm(_) => {
+                return Err(ErrorKind::NotSupported(
+                    "`DrmDisplayHandle` must be used with `egl::display::Display::with_device()`",
+                )
+                .into())
             },
             RawDisplayHandle::Windows(..) if extensions.contains("EGL_ANGLE_platform_angle") => {
                 // Only CreateWindowSurface appears to work with Angle.
@@ -352,11 +392,6 @@ impl Display {
             },
         };
 
-        // Be explicit here.
-        if display.is_null() {
-            display = egl::DEFAULT_DISPLAY as *mut _;
-        }
-
         // Push at the end so we can pop it on failure
         let mut has_display_reference = extensions.contains("EGL_KHR_display_reference");
         if has_display_reference {
@@ -404,11 +439,20 @@ impl Display {
     }
 
     fn get_display(egl: &Egl, display: RawDisplayHandle) -> Result<EglDisplay> {
-        let mut display = match display {
-            RawDisplayHandle::Gbm(handle) => handle.gbm_device,
-            #[cfg(x11_platform)]
-            RawDisplayHandle::Xlib(handle) => handle.display,
-            RawDisplayHandle::Android(_) => egl::DEFAULT_DISPLAY as *mut _,
+        let display = match display {
+            RawDisplayHandle::Gbm(handle) => handle.gbm_device.as_ptr(),
+            RawDisplayHandle::Drm(_) => {
+                return Err(ErrorKind::NotSupported(
+                    "`DrmDisplayHandle` must be used with `egl::display::Display::with_device()`",
+                )
+                .into())
+            },
+            RawDisplayHandle::Xlib(XlibDisplayHandle { display, .. }) => {
+                display.map_or(egl::DEFAULT_DISPLAY as *mut _, |d| d.as_ptr())
+            },
+            RawDisplayHandle::Android(_) | RawDisplayHandle::Ohos(_) => {
+                egl::DEFAULT_DISPLAY as *mut _
+            },
             _ => {
                 return Err(
                     ErrorKind::NotSupported("provided display handle is not supported").into()
@@ -416,10 +460,6 @@ impl Display {
             },
         };
 
-        if display.is_null() {
-            display = egl::DEFAULT_DISPLAY as *mut _;
-        }
-
         let display = unsafe { egl.GetDisplay(display) };
         Self::check_display_error(display).map(EglDisplay::Legacy)
     }
@@ -481,6 +521,26 @@ impl Display {
             Version::new(major as u8, minor as u8)
         };
 
+        let display = match display {
+            // `eglGetPlatformDisplay` and `GetPlatformDisplayEXT` aren't really differentiated,
+            // we must check if the version of the initialized display is not sensible for the
+            // EglDisplay type and downgrade it if so.
+            EglDisplay::Khr(display) if version <= Version { major: 1, minor: 4 } => {
+                let client_extensions = CLIENT_EXTENSIONS.get().unwrap();
+                if client_extensions.contains("EGL_EXT_platform_base")
+                    && (version == Version { major: 1, minor: 4 })
+                {
+                    // `EGL_EXT_platform_base` requires EGL 1.4 per specification; we cannot safely
+                    // presume that an `Ext` display would be valid for older versions.
+                    EglDisplay::Ext(display)
+                } else {
+                    EglDisplay::Legacy(display)
+                }
+            },
+            // We do not do anything otherwise.
+            display => display,
+        };
+
         // Load extensions.
         let display_extensions = get_extensions(egl, *display);
         let features = Self::extract_display_features(&display_extensions, version);
