@@ -33,7 +33,7 @@ impl Display {
         template: ConfigTemplate,
     ) -> Result<Box<dyn Iterator<Item = Config> + '_>> {
         let hwnd = match template.native_window {
-            Some(RawWindowHandle::Win32(window_handle)) => window_handle.hwnd as _,
+            Some(RawWindowHandle::Win32(window_handle)) => window_handle.hwnd.get() as _,
             _ => 0,
         };
         let hdc = unsafe { gdi::GetDC(hwnd) };
@@ -110,7 +110,7 @@ impl Display {
             cDepthBits: template.depth_size,
             cStencilBits: template.stencil_size,
             cAuxBuffers: 0,
-            iLayerType: gl::PFD_MAIN_PLANE,
+            iLayerType: gl::PFD_MAIN_PLANE as u8,
             bReserved: 0,
             dwLayerMask: 0,
             dwVisibleMask: 0,
@@ -237,6 +237,11 @@ impl Display {
             attrs.push(1);
         }
 
+        if template.config_surface_types.contains(ConfigSurfaceTypes::PBUFFER) {
+            attrs.push(wgl_extra::DRAW_TO_PBUFFER_ARB as c_int);
+            attrs.push(1);
+        }
+
         if template.transparency {
             attrs.push(wgl_extra::TRANSPARENT_ARB as c_int);
             attrs.push(1);
@@ -289,7 +294,7 @@ impl Config {
     /// The `raw_window_handle` should point to a valid value.
     pub unsafe fn apply_on_native_window(&self, raw_window_handle: &RawWindowHandle) -> Result<()> {
         let hdc = match raw_window_handle {
-            RawWindowHandle::Win32(window) => unsafe { gdi::GetDC(window.hwnd as _) },
+            RawWindowHandle::Win32(window) => unsafe { gdi::GetDC(window.hwnd.get() as _) },
             _ => return Err(ErrorKind::BadNativeWindow.into()),
         };
 
@@ -517,7 +522,7 @@ pub(crate) fn choose_dummy_pixel_format(hdc: HDC) -> Result<(i32, PIXELFORMATDES
         cDepthBits: 24,
         cStencilBits: 8,
         cAuxBuffers: 0,
-        iLayerType: gl::PFD_MAIN_PLANE,
+        iLayerType: gl::PFD_MAIN_PLANE as u8,
         bReserved: 0,
         dwLayerMask: 0,
         dwVisibleMask: 0,
