@@ -34,6 +34,9 @@ pub trait GlContext: Sealed {
     ///
     /// The returned value's [`Version`] will always be `None`.
     fn context_api(&self) -> ContextApi;
+
+    /// Get the [`Priority`] used by the context.
+    fn priority(&self) -> Priority;
 }
 
 /// A trait to group common not current operations.
@@ -51,7 +54,7 @@ pub trait NotCurrentGlContext: Sealed {
 
     /// Make [`Self::Surface`] on the calling thread producing the
     /// [`Self::PossiblyCurrentContext`] indicating that the context could
-    /// be current on the theard.
+    /// be current on the thread.
     ///
     /// # Platform specific
     ///
@@ -97,6 +100,11 @@ pub trait PossiblyCurrentGlContext: Sealed {
     /// - **macOS: this will block if your main thread is blocked.**
     fn make_not_current(self) -> Result<Self::NotCurrentContext>;
 
+    /// Make the context not current to the current thread. If you need to
+    /// send the context to another thread, use [`Self::make_not_current`]
+    /// instead.
+    fn make_not_current_in_place(&self) -> Result<()>;
+
     /// Make [`Self::Surface`] current on the calling thread.
     ///
     /// # Platform specific
@@ -201,6 +209,24 @@ impl ContextAttributesBuilder {
         self
     }
 
+    /// Set the priority hint, which might not be honored if the API does not
+    /// support it, if there are constraints on the number of high priority
+    /// contexts available in the system, or system policy limits access to
+    /// high priority contexts to appropriate system privilege level the
+    /// context creation may fail.
+    ///
+    /// By default no priority is specified, which corresponds to
+    /// [`Priority::Medium`].
+    ///
+    /// # Api specific
+    ///
+    /// - **WGL/GLX:** not implemented.
+    /// - **CGL:** not supported.
+    pub fn with_priority(mut self, priority: Priority) -> Self {
+        self.attributes.priority = Some(priority);
+        self
+    }
+
     /// Build the context attributes.
     ///
     /// The `raw_window_handle` isn't required and here for WGL compatibility.
@@ -228,6 +254,8 @@ pub struct ContextAttributes {
 
     pub(crate) api: Option<ContextApi>,
 
+    pub(crate) priority: Option<Priority>,
+
     pub(crate) shared_context: Option<RawContext>,
 
     pub(crate) raw_window_handle: Option<RawWindowHandle>,
@@ -442,6 +470,10 @@ impl GlContext for NotCurrentContext {
     fn context_api(&self) -> ContextApi {
         gl_api_dispatch!(self; Self(context) => context.context_api())
     }
+
+    fn priority(&self) -> Priority {
+        gl_api_dispatch!(self; Self(context) => context.priority())
+    }
 }
 
 impl GetGlConfig for NotCurrentContext {
@@ -517,6 +549,10 @@ impl PossiblyCurrentGlContext for PossiblyCurrentContext {
         )
     }
 
+    fn make_not_current_in_place(&self) -> Result<()> {
+        Ok(gl_api_dispatch!(self; Self(context) => context.make_not_current_in_place()?))
+    }
+
     fn make_current<T: SurfaceTypeTrait>(&self, surface: &Self::Surface<T>) -> Result<()> {
         match (self, surface) {
             #[cfg(egl_backend)]
@@ -562,6 +598,10 @@ impl GlContext for PossiblyCurrentContext {
     fn context_api(&self) -> ContextApi {
         gl_api_dispatch!(self; Self(context) => context.context_api())
     }
+
+    fn priority(&self) -> Priority {
+        gl_api_dispatch!(self; Self(context) => context.priority())
+    }
 }
 
 impl GetGlConfig for PossiblyCurrentContext {
@@ -608,6 +648,25 @@ pub enum RawContext {
     Cgl(*const ffi::c_void),
 }
 
+/// Priority hint
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
+pub enum Priority {
+    /// Lowest priority, contexts using this priority give way for most other
+    /// contexts.
+    Low,
+    /// Default priority.
+    #[default]
+    Medium,
+    /// High priority is usually required for VR applications.
+    High,
+    /// Realtime priority contexts are executed immediately and preempt any
+    /// current context running.
+    ///
+    /// When such context is not supported, [`Priority::High`] will be requested
+    /// instead.
+    Realtime,
+}
+
 /// Pick `GlProfile` and `Version` based on the provided params.
 #[cfg(any(egl_backend, glx_backend, wgl_backend))]
 pub(crate) fn pick_profile(
