@@ -61,7 +61,16 @@ pub trait GlDisplay: Sealed {
     /// Some platforms use [`RawWindowHandle`] for context creation, so it must
     /// point to a valid object.
     ///
+    /// # Platform-specific
+    ///
+    /// - **Wayland:** this call may latch the underlying back buffer of the
+    ///   currently active context (will do with mesa drivers), meaning that all
+    ///   resize operations will apply to it after the next
+    ///   [`GlSurface::swap_buffers`]. To workaround this behavior the current
+    ///   context should be made [`not current`].
+    ///
     /// [`RawWindowHandle`]: raw_window_handle::RawWindowHandle
+    /// [`not current`]: crate::context::PossiblyCurrentGlContext::make_not_current
     unsafe fn create_context(
         &self,
         config: &Self::Config,
@@ -110,9 +119,9 @@ pub trait GlDisplay: Sealed {
     ///
     /// # Api-specific
     ///
-    /// **WGL:** - To load all the functions you must have a current context on
-    /// the calling thread, otherwise only limited set of functions will be
-    /// loaded.
+    /// - **WGL:** to load all the functions you must have a current context on
+    ///   the calling thread, otherwise only a limited set of functions will be
+    ///   loaded.
     fn get_proc_address(&self, addr: &CStr) -> *const ffi::c_void;
 
     /// Helper to obtain the information about the underlying display.
@@ -143,8 +152,8 @@ pub trait GetDisplayExtensions: Sealed {
     ///
     /// # Api-specific
     ///
-    /// **WGL:** - To have extensions loaded, `raw_window_handle` must be used
-    /// when creating display.
+    /// - **WGL:** to have extensions loaded, `raw_window_handle` must be used
+    ///   when creating the display.
     fn extensions(&self) -> &HashSet<&'static str>;
 }
 
@@ -262,19 +271,19 @@ impl GlDisplay for Display {
         match self {
             #[cfg(egl_backend)]
             Self::Egl(display) => unsafe {
-                Ok(Box::new(display.find_configs(template)?.into_iter().map(Config::Egl)))
+                Ok(Box::new(display.find_configs(template)?.map(Config::Egl)))
             },
             #[cfg(glx_backend)]
             Self::Glx(display) => unsafe {
-                Ok(Box::new(display.find_configs(template)?.into_iter().map(Config::Glx)))
+                Ok(Box::new(display.find_configs(template)?.map(Config::Glx)))
             },
             #[cfg(wgl_backend)]
             Self::Wgl(display) => unsafe {
-                Ok(Box::new(display.find_configs(template)?.into_iter().map(Config::Wgl)))
+                Ok(Box::new(display.find_configs(template)?.map(Config::Wgl)))
             },
             #[cfg(cgl_backend)]
             Self::Cgl(display) => unsafe {
-                Ok(Box::new(display.find_configs(template)?.into_iter().map(Config::Cgl)))
+                Ok(Box::new(display.find_configs(template)?.map(Config::Cgl)))
             },
         }
     }
@@ -417,6 +426,11 @@ pub enum DisplayApiPreference {
     ///
     /// But despite this issues it should be preferred on at least Linux over
     /// GLX, given that GLX is phasing away.
+    ///
+    /// # Platform-specific
+    ///
+    /// **Windows:** ANGLE can be used if `libEGL.dll` and `libGLESv2.dll` are
+    ///              in the library search path.
     #[cfg(egl_backend)]
     Egl,
 
@@ -513,6 +527,7 @@ impl fmt::Debug for DisplayApiPreference {
 
 bitflags! {
     /// The features and extensions supported by the [`Display`].
+    #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
     pub struct DisplayFeatures: u32 {
         /// The display supports creating [`robust`] context.
         ///
