@@ -9,6 +9,10 @@
 //! The initialization starts by loading and connecting to the platform's
 //! graphics Api when creating a [`display`]. This object is used to create all
 //! the OpenGL objects, such as [`config`], [`context`], and [`surface`].
+//!
+//! ## Environment variables
+//!
+//! `GLUTIN_WGL_OPENGL_DLL` - change the name of the OpenGL DLL to load.
 
 #![deny(rust_2018_idioms)]
 #![deny(rustdoc::broken_intra_doc_links)]
@@ -17,7 +21,7 @@
 #![deny(clippy::all)]
 #![deny(missing_debug_implementations)]
 #![deny(missing_docs)]
-#![cfg_attr(feature = "cargo-clippy", deny(warnings))]
+#![cfg_attr(clippy, deny(warnings))]
 #![cfg_attr(docsrs, feature(doc_auto_cfg))]
 
 #[cfg(all(not(egl_backend), not(glx_backend), not(wgl_backend), not(cgl_backend)))]
